function x = jacobi_opt(A, b)
%JACOBI_OPT Soluzione di un sistema lineare usando il metodo di Jacobi.
%
% X = JACOBI_OPT(A, B) risolve il sistema lineare AX = B usando il metodo di
%     Jacobi sulla matrice A. Vengono effettuate al massimo 100 iterazioni,
%     e il metodo si ferma se il residue scende sotto 1e-8. 

max_iter = 100;

m = size(A, 1);
n = size(A, 2);

if m ~= n
    error('La matrice A non è quadrata');
end

if size(b, 1) ~= n
    error('Il vettore b non ha dimensioni compatibili con A');
end

x = b;

for j = 1 : max_iter
    % Iterazione j
    
    x_new = x;
    
    for i = 1 : n        
        % Questa e le istruzioni seguenti sono scritte per funzionare anche
        % quando b ha più colonne, e dunque vengono risolti più sistemi
        % lineari in un colpo solo. 
        x_new(i, :) = b(i, :);
        
        for s = 1 : n
            if s ~= i
                x_new(i, :) = x_new(i, :) - A(i, s) * x(s, :);
            end
        end
        x_new(i, :) = A(i,i) \ x_new(i, :);
    end
    
    x = x_new;
    
    % Stampiamo il residuo
    res = norm(A * x - b);    
    fprintf('Jacobi :: Iterazione %d :: Residuo %e\n', j, res);
    
    if res < 1e-8
        break
    end
end
